@@ -31,8 +31,9 @@ import (
 	"k8s.io/apimachinery/pkg/runtime/schema"
 	"k8s.io/apimachinery/pkg/types"
 
-	v1 "kubevirt.io/client-go/api/v1"
+	v1 "kubevirt.io/api/core/v1"
 	"kubevirt.io/client-go/precond"
+	cdiv1 "kubevirt.io/containerized-data-importer-api/pkg/apis/core/v1beta1"
 )
 
 // For versioning of the virt-handler and -launcher communication,
@@ -91,6 +92,10 @@ const (
 	ReasonPausedPostcopyFailed StateChangeReason = "PostcopyFailed"
 
 	UserAliasPrefix = "ua-"
+
+	FSThawed      = "thawed"
+	FSFrozen      = "frozen"
+	SchedulerFIFO = "fifo"
 )
 
 // +k8s:deepcopy-gen:interfaces=k8s.io/apimachinery/pkg/runtime.Object
@@ -102,10 +107,11 @@ type Domain struct {
 }
 
 type DomainStatus struct {
-	Status     LifeCycle
-	Reason     StateChangeReason
-	Interfaces []InterfaceStatus
-	OSInfo     GuestOSInfo
+	Status         LifeCycle
+	Reason         StateChangeReason
+	Interfaces     []InterfaceStatus
+	OSInfo         GuestOSInfo
+	FSFreezeStatus FSFreeze
 }
 
 type DomainSysInfo struct {
@@ -131,6 +137,7 @@ type InterfaceStatus struct {
 	Ip            string
 	IPs           []string
 	InterfaceName string
+	InfoSource    string
 }
 
 type Timezone struct {
@@ -138,6 +145,10 @@ type Timezone struct {
 	Offset int
 }
 
+type FSFreeze struct {
+	Status string
+}
+
 type Filesystem struct {
 	Name       string
 	Mountpoint string
@@ -154,8 +165,9 @@ type User struct {
 
 // DomainGuestInfo represent guest agent info for specific domain
 type DomainGuestInfo struct {
-	Interfaces []InterfaceStatus
-	OSInfo     *GuestOSInfo
+	Interfaces     []InterfaceStatus
+	OSInfo         *GuestOSInfo
+	FSFreezeStatus *FSFreeze
 }
 
 // +k8s:deepcopy-gen:interfaces=k8s.io/apimachinery/pkg/runtime.Object
@@ -169,40 +181,59 @@ type DomainList struct {
 // tagged, and they must correspond to the libvirt domain as described in
 // https://libvirt.org/formatdomain.html.
 type DomainSpec struct {
-	XMLName       xml.Name       `xml:"domain"`
-	Type          string         `xml:"type,attr"`
-	XmlNS         string         `xml:"xmlns:qemu,attr,omitempty"`
-	Name          string         `xml:"name"`
-	UUID          string         `xml:"uuid,omitempty"`
-	Memory        Memory         `xml:"memory"`
-	MemoryBacking *MemoryBacking `xml:"memoryBacking,omitempty"`
-	OS            OS             `xml:"os"`
-	SysInfo       *SysInfo       `xml:"sysinfo,omitempty"`
-	Devices       Devices        `xml:"devices"`
-	Clock         *Clock         `xml:"clock,omitempty"`
-	Resource      *Resource      `xml:"resource,omitempty"`
-	QEMUCmd       *Commandline   `xml:"qemu:commandline,omitempty"`
-	Metadata      Metadata       `xml:"metadata,omitempty"`
-	Features      *Features      `xml:"features,omitempty"`
-	CPU           CPU            `xml:"cpu"`
-	VCPU          *VCPU          `xml:"vcpu"`
-	CPUTune       *CPUTune       `xml:"cputune"`
-	IOThreads     *IOThreads     `xml:"iothreads,omitempty"`
+	XMLName        xml.Name        `xml:"domain"`
+	Type           string          `xml:"type,attr"`
+	XmlNS          string          `xml:"xmlns:qemu,attr,omitempty"`
+	Name           string          `xml:"name"`
+	UUID           string          `xml:"uuid,omitempty"`
+	Memory         Memory          `xml:"memory"`
+	MemoryBacking  *MemoryBacking  `xml:"memoryBacking,omitempty"`
+	OS             OS              `xml:"os"`
+	SysInfo        *SysInfo        `xml:"sysinfo,omitempty"`
+	Devices        Devices         `xml:"devices"`
+	Clock          *Clock          `xml:"clock,omitempty"`
+	Resource       *Resource       `xml:"resource,omitempty"`
+	QEMUCmd        *Commandline    `xml:"qemu:commandline,omitempty"`
+	Metadata       Metadata        `xml:"metadata,omitempty"`
+	Features       *Features       `xml:"features,omitempty"`
+	CPU            CPU             `xml:"cpu"`
+	VCPU           *VCPU           `xml:"vcpu"`
+	CPUTune        *CPUTune        `xml:"cputune"`
+	NUMATune       *NUMATune       `xml:"numatune"`
+	IOThreads      *IOThreads      `xml:"iothreads,omitempty"`
+	LaunchSecurity *LaunchSecurity `xml:"launchSecurity,omitempty"`
 }
 
 type CPUTune struct {
-	VCPUPin     []CPUTuneVCPUPin     `xml:"vcpupin"`
-	IOThreadPin []CPUTuneIOThreadPin `xml:"iothreadpin,omitempty"`
-	EmulatorPin *CPUEmulatorPin      `xml:"emulatorpin"`
+	VCPUPin       []CPUTuneVCPUPin     `xml:"vcpupin"`
+	IOThreadPin   []CPUTuneIOThreadPin `xml:"iothreadpin,omitempty"`
+	EmulatorPin   *CPUEmulatorPin      `xml:"emulatorpin"`
+	VCPUScheduler *VCPUScheduler       `xml:"vcpusched,omitempty"`
+}
+
+type NUMATune struct {
+	Memory   NumaTuneMemory `xml:"memory"`
+	MemNodes []MemNode      `xml:"memnode"`
+}
+
+type MemNode struct {
+	CellID  uint32 `xml:"cellid,attr"`
+	Mode    string `xml:"mode,attr"`
+	NodeSet string `xml:"nodeset,attr"`
+}
+
+type NumaTuneMemory struct {
+	Mode    string `xml:"mode,attr"`
+	NodeSet string `xml:"nodeset,attr"`
 }
 
 type CPUTuneVCPUPin struct {
-	VCPU   uint   `xml:"vcpu,attr"`
+	VCPU   uint32 `xml:"vcpu,attr"`
 	CPUSet string `xml:"cpuset,attr"`
 }
 
 type CPUTuneIOThreadPin struct {
-	IOThread uint   `xml:"iothread,attr"`
+	IOThread uint32 `xml:"iothread,attr"`
 	CPUSet   string `xml:"cpuset,attr"`
 }
 
@@ -210,6 +241,12 @@ type CPUEmulatorPin struct {
 	CPUSet string `xml:"cpuset,attr"`
 }
 
+type VCPUScheduler struct {
+	Scheduler string `xml:"scheduler,attr"`
+	Priority  uint   `xml:"priority,attr"`
+	VCPUs     string `xml:"vcpus,attr"`
+}
+
 type VCPU struct {
 	Placement string `xml:"placement,attr"`
 	CPUs      uint32 `xml:",chardata"`
@@ -230,7 +267,7 @@ type NUMA struct {
 type NUMACell struct {
 	ID           string `xml:"id,attr"`
 	CPUs         string `xml:"cpus,attr"`
-	Memory       string `xml:"memory,attr,omitempty"`
+	Memory       uint64 `xml:"memory,attr,omitempty"`
 	Unit         string `xml:"unit,attr,omitempty"`
 	MemoryAccess string `xml:"memAccess,attr,omitempty"`
 }
@@ -253,6 +290,7 @@ type Features struct {
 	SMM        *FeatureEnabled    `xml:"smm,omitempty"`
 	KVM        *FeatureKVM        `xml:"kvm,omitempty"`
 	PVSpinlock *FeaturePVSpinlock `xml:"pvspinlock,omitempty"`
+	PMU        *FeatureState      `xml:"pmu,omitempty"`
 }
 
 type FeatureHyperv struct {
@@ -299,7 +337,8 @@ type FeatureState struct {
 }
 
 type FeatureKVM struct {
-	Hidden *FeatureState `xml:"hidden,omitempty"`
+	Hidden        *FeatureState `xml:"hidden,omitempty"`
+	HintDedicated *FeatureState `xml:"hint-dedicated,omitempty"`
 }
 
 type Metadata struct {
@@ -362,9 +401,21 @@ type Memory struct {
 
 // MemoryBacking mirroring libvirt XML under https://libvirt.org/formatdomain.html#elementsMemoryBacking
 type MemoryBacking struct {
-	HugePages *HugePages           `xml:"hugepages,omitempty"`
-	Source    *MemoryBackingSource `xml:"source,omitempty"`
-	Access    *MemoryBackingAccess `xml:"access,omitempty"`
+	HugePages    *HugePages           `xml:"hugepages,omitempty"`
+	Source       *MemoryBackingSource `xml:"source,omitempty"`
+	Access       *MemoryBackingAccess `xml:"access,omitempty"`
+	Allocation   *MemoryAllocation    `xml:"allocation,omitempty"`
+	NoSharePages *NoSharePages        `xml:"nosharepages,omitempty"`
+}
+
+type MemoryAllocationMode string
+
+const (
+	MemoryAllocationModeImmediate MemoryAllocationMode = "immediate"
+)
+
+type MemoryAllocation struct {
+	Mode MemoryAllocationMode `xml:"mode,attr"`
 }
 
 type MemoryBackingSource struct {
@@ -378,14 +429,18 @@ type HugePages struct {
 
 // HugePage mirroring libvirt XML under hugepages
 type HugePage struct {
-	Size string `xml:"size,attr"`
-	Unit string `xml:"unit,attr"`
+	Size    string `xml:"size,attr"`
+	Unit    string `xml:"unit,attr"`
+	NodeSet string `xml:"nodeset,attr"`
 }
 
 type MemoryBackingAccess struct {
 	Mode string `xml:"mode,attr"`
 }
 
+type NoSharePages struct {
+}
+
 type Devices struct {
 	Emulator    string             `xml:"emulator,omitempty"`
 	Interfaces  []Interface        `xml:"interface"`
@@ -402,6 +457,21 @@ type Devices struct {
 	Watchdog    *Watchdog          `xml:"watchdog,omitempty"`
 	Rng         *Rng               `xml:"rng,omitempty"`
 	Filesystems []FilesystemDevice `xml:"filesystem,omitempty"`
+	Redirs      []RedirectedDevice `xml:"redirdev,omitempty"`
+	SoundCards  []SoundCard        `xml:"sound,omitempty"`
+}
+
+// RedirectedDevice describes a device to be redirected
+// See: https://libvirt.org/formatdomain.html#redirected-devices
+type RedirectedDevice struct {
+	Type   string                 `xml:"type,attr"`
+	Bus    string                 `xml:"bus,attr"`
+	Source RedirectedDeviceSource `xml:"source"`
+}
+
+type RedirectedDeviceSource struct {
+	Mode string `xml:"mode,attr"`
+	Path string `xml:"path,attr"`
 }
 
 type FilesystemDevice struct {
@@ -462,6 +532,8 @@ type HostDevice struct {
 	Model     string           `xml:"model,attr,omitempty"`
 	Address   *Address         `xml:"address,emitempty"`
 	Alias     *Alias           `xml:"alias,omitempty"`
+	Display   string           `xml:"display,attr,omitempty"`
+	RamFB     string           `xml:"ramfb,attr,omitempty"`
 }
 
 type HostDeviceSource struct {
@@ -486,7 +558,9 @@ type Controller struct {
 
 // BEGIN ControllerDriver
 type ControllerDriver struct {
-	IOThread *uint `xml:"iothread,attr,omitempty"`
+	IOThread *uint  `xml:"iothread,attr,omitempty"`
+	Queues   *uint  `xml:"queues,attr,omitempty"`
+	IOMMU    string `xml:"iommu,attr,omitempty"`
 }
 
 // END ControllerDriver
@@ -494,21 +568,24 @@ type ControllerDriver struct {
 // BEGIN Disk -----------------------------
 
 type Disk struct {
-	Device       string        `xml:"device,attr"`
-	Snapshot     string        `xml:"snapshot,attr,omitempty"`
-	Type         string        `xml:"type,attr"`
-	Source       DiskSource    `xml:"source"`
-	Target       DiskTarget    `xml:"target"`
-	Serial       string        `xml:"serial,omitempty"`
-	Driver       *DiskDriver   `xml:"driver,omitempty"`
-	ReadOnly     *ReadOnly     `xml:"readonly,omitempty"`
-	Auth         *DiskAuth     `xml:"auth,omitempty"`
-	Alias        *Alias        `xml:"alias,omitempty"`
-	BackingStore *BackingStore `xml:"backingStore,omitempty"`
-	BootOrder    *BootOrder    `xml:"boot,omitempty"`
-	Address      *Address      `xml:"address,omitempty"`
-	Model        string        `xml:"model,attr,omitempty"`
-	BlockIO      *BlockIO      `xml:"blockio,omitempty"`
+	Device             string         `xml:"device,attr"`
+	Snapshot           string         `xml:"snapshot,attr,omitempty"`
+	Type               string         `xml:"type,attr"`
+	Source             DiskSource     `xml:"source"`
+	Target             DiskTarget     `xml:"target"`
+	Serial             string         `xml:"serial,omitempty"`
+	Driver             *DiskDriver    `xml:"driver,omitempty"`
+	ReadOnly           *ReadOnly      `xml:"readonly,omitempty"`
+	Auth               *DiskAuth      `xml:"auth,omitempty"`
+	Alias              *Alias         `xml:"alias,omitempty"`
+	BackingStore       *BackingStore  `xml:"backingStore,omitempty"`
+	BootOrder          *BootOrder     `xml:"boot,omitempty"`
+	Address            *Address       `xml:"address,omitempty"`
+	Model              string         `xml:"model,attr,omitempty"`
+	BlockIO            *BlockIO       `xml:"blockio,omitempty"`
+	FilesystemOverhead *cdiv1.Percent `xml:"filesystemOverhead,omitempty"`
+	Capacity           *int64         `xml:"capacity,omitempty"`
+	ExpandDisksEnabled bool           `xml:"expandDisksEnabled,omitempty"`
 }
 
 type DiskAuth struct {
@@ -540,14 +617,15 @@ type DiskTarget struct {
 }
 
 type DiskDriver struct {
-	Cache       string `xml:"cache,attr,omitempty"`
-	ErrorPolicy string `xml:"error_policy,attr,omitempty"`
-	IO          string `xml:"io,attr,omitempty"`
-	Name        string `xml:"name,attr"`
-	Type        string `xml:"type,attr"`
-	IOThread    *uint  `xml:"iothread,attr,omitempty"`
-	Queues      *uint  `xml:"queues,attr,omitempty"`
-	Discard     string `xml:"discard,attr,omitempty"`
+	Cache       string      `xml:"cache,attr,omitempty"`
+	ErrorPolicy string      `xml:"error_policy,attr,omitempty"`
+	IO          v1.DriverIO `xml:"io,attr,omitempty"`
+	Name        string      `xml:"name,attr"`
+	Type        string      `xml:"type,attr"`
+	IOThread    *uint       `xml:"iothread,attr,omitempty"`
+	Queues      *uint       `xml:"queues,attr,omitempty"`
+	Discard     string      `xml:"discard,attr,omitempty"`
+	IOMMU       string      `xml:"iommu,attr,omitempty"`
 }
 
 type DiskSourceHost struct {
@@ -636,6 +714,7 @@ type Interface struct {
 type InterfaceDriver struct {
 	Name   string `xml:"name,attr"`
 	Queues *uint  `xml:"queues,attr,omitempty"`
+	IOMMU  string `xml:"iommu,attr,omitempty"`
 }
 
 type LinkState struct {
@@ -807,7 +886,16 @@ type Entry struct {
 }
 
 //END OS --------------------
+//BEGIN LaunchSecurity --------------------
 
+type LaunchSecurity struct {
+	Type            string `xml:"type,attr"`
+	Cbitpos         string `xml:"cbitpos,omitempty"`
+	ReducedPhysBits string `xml:"reducedPhysBits,omitempty"`
+	Policy          string `xml:"policy,omitempty"`
+}
+
+//END LaunchSecurity --------------------
 //BEGIN Clock --------------------
 
 type Clock struct {
@@ -822,6 +910,7 @@ type Timer struct {
 	TickPolicy string `xml:"tickpolicy,attr,omitempty"`
 	Present    string `xml:"present,attr,omitempty"`
 	Track      string `xml:"track,attr,omitempty"`
+	Frequency  string `xml:"frequency,attr,omitempty"`
 }
 
 //END Clock --------------------
@@ -849,10 +938,16 @@ type ChannelSource struct {
 
 //END Channel --------------------
 
+//BEGIN Sound -------------------
+
+type SoundCard struct {
+	Alias *Alias `xml:"alias,omitempty"`
+	Model string `xml:"model,attr"`
+}
+
+//END Sound -------------------
+
 //BEGIN Video -------------------
-/*
-<graphics autoport="yes" defaultMode="secure" listen="0" passwd="*****" passwdValidTo="1970-01-01T00:00:01" port="-1" tlsPort="-1" type="spice" />
-*/
 
 type Video struct {
 	Model VideoModel `xml:"model"`
@@ -902,9 +997,14 @@ type Stats struct {
 }
 
 type MemBalloon struct {
-	Model   string   `xml:"model,attr"`
-	Stats   *Stats   `xml:"stats,omitempty"`
-	Address *Address `xml:"address,emitempty"`
+	Model   string            `xml:"model,attr"`
+	Stats   *Stats            `xml:"stats,omitempty"`
+	Address *Address          `xml:"address,emitempty"`
+	Driver  *MemBalloonDriver `xml:"driver,emitempty"`
+}
+
+type MemBalloonDriver struct {
+	IOMMU string `xml:"iommu,attr,omitempty"`
 }
 
 type Watchdog struct {
@@ -921,6 +1021,11 @@ type Rng struct {
 	// Backend specifies the source of entropy to be used
 	Backend *RngBackend `xml:"backend,omitempty"`
 	Address *Address    `xml:"address,emitempty"`
+	Driver  *RngDriver  `xml:"driver,emitempty"`
+}
+
+type RngDriver struct {
+	IOMMU string `xml:"iommu,attr,omitempty"`
 }
 
 // RngRate sets the limiting factor how to read from entropy source
